/*
 * timer.h
 *
 *  Created on: 20.7.2010
 *      Author: M.inko
 */
#ifndef TIMER_H
#define TIMER_H

#include <avr/io.h>
#include "avrlibdefs.h"
#include "avrlibtypes.h"

#define TIMER_CLK_STOP			0x00	///< Timer Stopped
#define TIMER_CLK_DIV1			0x01	///< Timer clocked at F_CPU
#define TIMER_CLK_DIV8			0x02	///< Timer clocked at F_CPU/8
#define TIMER_CLK_DIV64			0x03	///< Timer clocked at F_CPU/64
#define TIMER_CLK_DIV256		0x04	///< Timer clocked at F_CPU/256
#define TIMER_CLK_DIV1024		0x05	///< Timer clocked at F_CPU/1024
#define TIMER_CLK_T_FALL		0x06	///< Timer clocked at T falling edge
#define TIMER_CLK_T_RISE		0x07	///< Timer clocked at T rising edge
#define TIMER_PRESCALE_MASK		0x07	///< Timer Prescaler Bit-Mask


void timer0SetPrescaler(BYTE prescale);		///< set timer1 prescaler


//input value is number of internal delay loops. each delay loop consumes 4 cycles of MCU
// the time resolution is dependent on the time the loop takes, tahat means on the F_CPU
// e.g. with 4Mhz and 4 cycles per loop, the resolution is 1 us
void delay_us_loops(WORD delay_loops);

//conversion macro. input value is not number of loops but more user-friendly: time in microseconds
//number of loops depends on F_CPU because duration of esch loop consumes different time with different F_CPU
//period of 1 MCU's cycle  = 1/F_CPU [s]
//period of 4 MCU's cycles (1 delay loop) = 4/F_CPU [s]
//num of required loops for given time [s]: time/(4/F_CPU) = (time*F_CPU)/4
//when time is in [us]: ((time_us*F_CPU)/4)/1000000 = (time_us*F_CPU)/4000000
//(double) conversions are for preprocessor only to get max precision. final result is cast to word

//max value of time_us for correct function has to result to max 65535 delay loops. the max time_us depends on F_CPU
//time_us = loops*4000000/F_CPU
//time_us_max = 65535*4000000/F_CPU
//
//F_CPU[MHz] | time_us_max
//---------------------
//    1      | 262140us
//    1.2    | 218450us
//    1.5    | 174760us
//    4      |  65535us
//    8      |  32767us
//   12      |  21845us
//   20      |  13107us
#define DELAY_US(time_us) delay_us_loops((WORD)((double)(time_us)*(double)F_CPU/4000000.0))

#endif
